function [t, r_history,t_history, Lodr,Lodt,Ls,epsilon]=TPW_Hu_for_Midori9(M,alpha,t0,Dt,End_time,Model,Loadtype,T_C)
%% Semi-analytical solution for TPW on rotating body whose Chandler period is much shorter than its dominant relaxation time (Earth-like).
%% by Haiyang X.S. Hu   02-05-2018
%% Output
% t: time step;  
% r_history: colatitude and longitude of the rotational axis (rad) in the body-fixed frame;
% t_history: colatitude and longitude of the tidal axis (rad) in the body-fixed frame;
% Lodr: length of the day change;
% Lodt: length of the day change for tidal axis;
% Ls: colatitude and longitude of the mass anomaly (rad) in the bulge-fixed frame; 
% epsilon: angels (rad) between the principle moment of inertia axes and the tidal/rotational axis. 
%% Input 
% M: mass of the massanomaly (kg);
% alpha: colatitude and longitude (degree) of the mass anomaly. 
% t0: Initial Loading time (ka) for the model to reach its desired hydrostatic state. It needs to be large enough for all the modes to relax for a model in hydrostatic equilibrium.
% Dt: step size (ka)
% End_time: simulation length (ka)
% Model: choice of the interior model Love numbers;
% Lodtype: 1 for Heaviside, 2 for Ramp type.
% T_C: tidal factor number. set to 3 for most of the icy moons.
%% Example
% To reproduce figure 1 in [Hu at al, 2018], the reorientation paths for the three Heaviside mass anomalies can be generated by commands:
%[t1, r_history1,t_history1, ~,~,Ls1]=TPW_Hu_for_Midori9(3e17,[15,-15],1e12,12,1.2e4,3,1,3);
%[t2, r_history2,t_history2, ~,~,Ls2]=TPW_Hu_for_Midori9(1.5e17,[15,-15],1e12,7,1.5e4,3,1,3);
%[t3, r_history3,t_history3, ~,~,Ls3]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,8,8e3,3,1,3);

% To reproduce figure 3 in [Hu at al, 2018], the reorientation paths for the seven ramp mass anomalies can be generated by commands:
% [t1, ~,~,~,~,Ls1,epsilon1]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,7,5e3,3,2,3);
% [t2,~,~, ~,~,Ls2]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,20,20e3,3,2,3);
% [t3,~,~, ~,~,Ls3]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,60,60e3,3,2,3);
% [t4,~,~, ~,~,Ls4]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,120,120e3,3,2,3);
% [t5,~,~, ~,~,Ls5]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,360,360e3,3,2,3);
% [t6,~,~, ~,~,Ls6]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,1000,1000e3,3,2,3);
% [t7,~,~, ~,~,Ls7]=TPW_Hu_for_Midori9(6e17,[15,-15],1e12,2000,2000e3,3,2,3);
%%

if length(alpha)~=2
    error('length of alpha must be 3.(vector)')
end
Size=size(alpha);
if Size(1)~=2
    alpha=alpha';
end
%%

if Model==1
    data=dlmread('modt_Pluto_2.txt');     
elseif Model==2
    data=dlmread('modt_Triton');   
elseif Model==3
    data=dlmread('modt_Triton_10.txt');
elseif Model==4
    data=dlmread('modt_Triton_10_modified.txt');
end
% maxdeg=data(1);
nlayer=data(2);
maxmod=data(3);
data=data(4+nlayer:end,:);
D=1;

Ke=data((maxmod+2)*(D-1)+1,3);
Kf=data((maxmod+2)*(D-1)+maxmod+2,3);
Ki=-data((maxmod+2)*(D-1)+(1:maxmod)+1,3).*data((maxmod+2)*(D-1)+(1:maxmod)+1,2);
Si=data((maxmod+2)*(D-1)+(1:maxmod)+1,2);
%%
GG=6.67384e-11;

%Omega=2*pi/(24)/60/60*Omega_portion;

ka=1000*365.25*24*3600;


if Model==1;
    I=3.7666e33; % Pluto
    R=1188e3;
    Omega=2*pi/(6.387230*24*3600);
%     T_C=3;
    T_C=0.3255;
else %Triton
    I=7.0920477e33; %moment of inertia of unperturbed Triton 
    R=1352e3;
    Omega= 2*pi/(((5*24+21)*60+2)*60+53); 
%     T_C=3;T_
end

Omega0=Omega; 
%% Load parameters

alpha=alpha*pi/180;
x=R*sin(alpha(1))*cos(alpha(2));
y=R*sin(alpha(1))*sin(alpha(2));
z=R*cos(alpha(1));
E=M*[y^2+z^2 -x*y -x*z; -y*x x^2+z^2 -y*z; -z*x -z*y x^2+y^2]; % Load matrix    
dE=0;

%% for testing
if Model==4
    E=E+R^5/3/GG*Omega^2*0.38758E-01*diag([-1/3,-1/3,2/3])-3*R^5/3/GG*Omega^2*0.38758E-01*diag([2/3,-1/3,-1/3]);
end
%%
Q_T=RotationM([pi/2,0,1],1); % rotational-tidal matrix
%%
t=zeros(1,3000);
t(1)=t0;

r_history=zeros(3,3000);
r_history(:,1)=[0,0,Omega]';
t_history=zeros(3,3000);
t_history(:,1)=[Omega*sqrt(T_C),0,0]';  %relationship for a tidally locked objects

Lodr=zeros(1,1000);
Lodr(1)=1;
Lodt=zeros(1,1000);
Lodt(1)=1;

epsilon=zeros(2,3000);
Ls=zeros(2,3000);
Ls(:,1)=alpha;

A=zeros(3);
dA=zeros(3);
B=zeros(3);
dB=zeros(3);

Ind=1;


while t(Ind)<t0+End_time
    if Ind==200
        aaaa=1;
    end
%     if t(Ind)>1e4+t0  %remove the mass anomaly after 1 milion year.
%        E=zeros(3);
%     end
    if Ind==1
        t(Ind+1)=t(Ind)+0.5;
    elseif Ind==2
        t(Ind+1)=t(Ind)+Dt-0.5;
    else
         t(Ind+1)=t(Ind)+Dt;
    end
    r_history(:,Ind+1)=r_history(:,Ind);
    t_history(:,Ind+1)=t_history(:,Ind);

    Coe=Ki./Si.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    A(1,1)=Coe*r_history(1,1)*r_history(1,1);
    A(1,2)=Coe*r_history(1,1)*r_history(2,1);
    A(1,3)=Coe*r_history(1,1)*r_history(3,1);
    A(2,2)=Coe*r_history(2,1)*r_history(2,1);
    A(2,3)=Coe*r_history(2,1)*r_history(3,1);
    A(3,3)=Coe*r_history(3,1)*r_history(3,1);
    
    B(1,1)=Coe*t_history(1,1)*t_history(1,1);
    B(1,2)=Coe*t_history(1,1)*t_history(2,1);
    B(1,3)=Coe*t_history(1,1)*t_history(3,1);
    B(2,2)=Coe*t_history(2,1)*t_history(2,1);
    B(2,3)=Coe*t_history(2,1)*t_history(3,1);
    B(3,3)=Coe*t_history(3,1)*t_history(3,1);
    
    Coe=Ki.*(exp(t(Ind+1)*Si)-exp((t(Ind+1)-t(1))*Si));
    Coe=sum(Coe);
    if Ind~=0
        dt=t(Ind+1)-t(Ind);
        dA(1,1)=Coe*r_history(1,1)*r_history(1,1)+Ke/dt*(r_history(1,Ind+1)*(r_history(1,Ind)-2*r_history(1,Ind+1))+r_history(1,Ind)*r_history(1,Ind+1));
        dA(1,2)=Coe*r_history(1,1)*r_history(2,1)+Ke/dt*(r_history(1,Ind+1)*(r_history(2,Ind)-2*r_history(2,Ind+1))+r_history(1,Ind)*r_history(2,Ind+1));
        dA(1,3)=Coe*r_history(1,1)*r_history(3,1)+Ke/dt*(r_history(1,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(1,Ind)*r_history(3,Ind+1));
        dA(2,2)=Coe*r_history(2,1)*r_history(2,1)+Ke/dt*(r_history(2,Ind+1)*(r_history(2,Ind)-2*r_history(2,Ind+1))+r_history(2,Ind)*r_history(2,Ind+1));
        dA(2,3)=Coe*r_history(2,1)*r_history(3,1)+Ke/dt*(r_history(2,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(2,Ind)*r_history(3,Ind+1));
        dA(3,3)=Coe*r_history(3,1)*r_history(3,1)+Ke/dt*(r_history(3,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(3,Ind)*r_history(3,Ind+1));
        
        dB(1,1)=Coe*t_history(1,1)*t_history(1,1)+Ke/dt*(t_history(1,Ind+1)*(t_history(1,Ind)-2*t_history(1,Ind+1))+t_history(1,Ind)*t_history(1,Ind+1));
        dB(1,2)=Coe*t_history(1,1)*t_history(2,1)+Ke/dt*(t_history(1,Ind+1)*(t_history(2,Ind)-2*t_history(2,Ind+1))+t_history(1,Ind)*t_history(2,Ind+1));
        dB(1,3)=Coe*t_history(1,1)*t_history(3,1)+Ke/dt*(t_history(1,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(1,Ind)*t_history(3,Ind+1));
        dB(2,2)=Coe*t_history(2,1)*t_history(2,1)+Ke/dt*(t_history(2,Ind+1)*(t_history(2,Ind)-2*t_history(2,Ind+1))+t_history(2,Ind)*t_history(2,Ind+1));
        dB(2,3)=Coe*t_history(2,1)*t_history(3,1)+Ke/dt*(t_history(2,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(2,Ind)*t_history(3,Ind+1));
        dB(3,3)=Coe*t_history(3,1)*t_history(3,1)+Ke/dt*(t_history(3,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(3,Ind)*t_history(3,Ind+1));
    else
        dA=zeros(3);
        dB=zeros(3);
    end
    for ii=1:maxmod
        if Ind>1
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),Ki(ii),Si(ii));
            A(1,1)=A(1,1)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),Ki(ii),Si(ii));
            A(1,2)=A(1,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            A(1,3)=A(1,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),Ki(ii),Si(ii));
            A(2,2)=A(2,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            A(2,3)=A(2,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            A(3,3)=A(3,3)+sum(temp);
            
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),Ki(ii),Si(ii));
            dA(1,1)=dA(1,1)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),Ki(ii),Si(ii));
            dA(1,2)=dA(1,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(1,1:Ind-1),r_history(1,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            dA(1,3)=dA(1,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),Ki(ii),Si(ii));
            dA(2,2)=dA(2,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(2,1:Ind-1),r_history(2,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            dA(2,3)=dA(2,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),r_history(3,1:Ind-1),r_history(3,2:Ind),Ki(ii),Si(ii));
            dA(3,3)=dA(3,3)+sum(temp);
            %
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),Ki(ii),Si(ii));
            B(1,1)=B(1,1)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),Ki(ii),Si(ii));
            B(1,2)=B(1,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            B(1,3)=B(1,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),Ki(ii),Si(ii));
            B(2,2)=B(2,2)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            B(2,3)=B(2,3)+sum(temp);
            temp=wwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            B(3,3)=B(3,3)+sum(temp);
            
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),Ki(ii),Si(ii));
            dB(1,1)=dB(1,1)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),Ki(ii),Si(ii));
            dB(1,2)=dB(1,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(1,1:Ind-1),t_history(1,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            dB(1,3)=dB(1,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),Ki(ii),Si(ii));
            dB(2,2)=dB(2,2)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(2,1:Ind-1),t_history(2,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            dB(2,3)=dB(2,3)+sum(temp);
            temp=dwwn(t(Ind+1),t(1:Ind-1),t(2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),t_history(3,1:Ind-1),t_history(3,2:Ind),Ki(ii),Si(ii));
            dB(3,3)=dB(3,3)+sum(temp);            
        end
    end

    
    
    AA=zeros(3);
    dAA=zeros(3);
    BB=zeros(3);
    dBB=zeros(3);
    
    Improvement1=100;
    Improvement2=100;
    Iter=1;
    
    Threshhold1=1e-8;
    Threshhold2=1e-8;  
    while (Improvement1>Threshhold1 ||Improvement2>Threshhold2)&& Iter<2000
        if Iter==400
           aaaa=1; 
           disp([num2str(Iter)  '    ' num2str(r_history(:,Ind+1)',10) '  '  num2str(t_history(:,Ind+1)',10)]);
        end
        AA(1,1)=A(1,1)+Ke*r_history(1,Ind+1)*r_history(1,Ind+1);
        AA(1,2)=A(1,2)+Ke*r_history(1,Ind+1)*r_history(2,Ind+1);
        AA(1,3)=A(1,3)+Ke*r_history(1,Ind+1)*r_history(3,Ind+1);
        AA(2,2)=A(2,2)+Ke*r_history(2,Ind+1)*r_history(2,Ind+1);
        AA(2,3)=A(2,3)+Ke*r_history(2,Ind+1)*r_history(3,Ind+1);
        AA(3,3)=A(3,3)+Ke*r_history(3,Ind+1)*r_history(3,Ind+1);
        
        BB(1,1)=B(1,1)+Ke*t_history(1,Ind+1)*t_history(1,Ind+1);
        BB(1,2)=B(1,2)+Ke*t_history(1,Ind+1)*t_history(2,Ind+1);
        BB(1,3)=B(1,3)+Ke*t_history(1,Ind+1)*t_history(3,Ind+1);
        BB(2,2)=B(2,2)+Ke*t_history(2,Ind+1)*t_history(2,Ind+1);
        BB(2,3)=B(2,3)+Ke*t_history(2,Ind+1)*t_history(3,Ind+1);
        BB(3,3)=B(3,3)+Ke*t_history(3,Ind+1)*t_history(3,Ind+1);
        
        dt=t(Ind+1)-t(Ind);
        if Ind~=1
            dAA(1,1)=dA(1,1)+Ke/dt*(r_history(1,Ind+1)*(r_history(1,Ind)-2*r_history(1,Ind+1))+r_history(1,Ind)*r_history(1,Ind+1));
            dAA(1,2)=dA(1,2)+Ke/dt*(r_history(1,Ind+1)*(r_history(2,Ind)-2*r_history(2,Ind+1))+r_history(1,Ind)*r_history(2,Ind+1));
            dAA(1,3)=dA(1,3)+Ke/dt*(r_history(1,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(1,Ind)*r_history(3,Ind+1));
            dAA(2,2)=dA(2,2)+Ke/dt*(r_history(2,Ind+1)*(r_history(2,Ind)-2*r_history(2,Ind+1))+r_history(2,Ind)*r_history(2,Ind+1));
            dAA(2,3)=dA(2,3)+Ke/dt*(r_history(2,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(2,Ind)*r_history(3,Ind+1));
            dAA(3,3)=dA(3,3)+Ke/dt*(r_history(3,Ind+1)*(r_history(3,Ind)-2*r_history(3,Ind+1))+r_history(3,Ind)*r_history(3,Ind+1));
            
            dBB(1,1)=dB(1,1)+Ke/dt*(t_history(1,Ind+1)*(t_history(1,Ind)-2*t_history(1,Ind+1))+t_history(1,Ind)*t_history(1,Ind+1));
            dBB(1,2)=dB(1,2)+Ke/dt*(t_history(1,Ind+1)*(t_history(2,Ind)-2*t_history(2,Ind+1))+t_history(1,Ind)*t_history(2,Ind+1));
            dBB(1,3)=dB(1,3)+Ke/dt*(t_history(1,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(1,Ind)*t_history(3,Ind+1));
            dBB(2,2)=dB(2,2)+Ke/dt*(t_history(2,Ind+1)*(t_history(2,Ind)-2*t_history(2,Ind+1))+t_history(2,Ind)*t_history(2,Ind+1));
            dBB(2,3)=dB(2,3)+Ke/dt*(t_history(2,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(2,Ind)*t_history(3,Ind+1));
            dBB(3,3)=dB(3,3)+Ke/dt*(t_history(3,Ind+1)*(t_history(3,Ind)-2*t_history(3,Ind+1))+t_history(3,Ind)*t_history(3,Ind+1));
        else
            dAA=dA;
            dBB=dB;
        end
        for ii=1:maxmod
            jj=Ind;
            AA(1,1)=AA(1,1)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(1,jj),r_history(1,jj+1),Ki(ii),Si(ii));
            AA(1,2)=AA(1,2)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(2,jj),r_history(2,jj+1),Ki(ii),Si(ii));
            AA(1,3)=AA(1,3)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));
            AA(2,2)=AA(2,2)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(2,jj),r_history(2,jj+1),r_history(2,jj),r_history(2,jj+1),Ki(ii),Si(ii));
            AA(2,3)=AA(2,3)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(2,jj),r_history(2,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));
            AA(3,3)=AA(3,3)+wwn(t(Ind+1),t(jj),t(jj+1),r_history(3,jj),r_history(3,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));
            
            dAA(1,1)=dAA(1,1)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(1,jj),r_history(1,jj+1),Ki(ii),Si(ii));
            dAA(1,2)=dAA(1,2)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(2,jj),r_history(2,jj+1),Ki(ii),Si(ii));
            dAA(1,3)=dAA(1,3)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(1,jj),r_history(1,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));
            dAA(2,2)=dAA(2,2)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(2,jj),r_history(2,jj+1),r_history(2,jj),r_history(2,jj+1),Ki(ii),Si(ii));
            dAA(2,3)=dAA(2,3)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(2,jj),r_history(2,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));
            dAA(3,3)=dAA(3,3)+dwwn(t(Ind+1),t(jj),t(jj+1),r_history(3,jj),r_history(3,jj+1),r_history(3,jj),r_history(3,jj+1),Ki(ii),Si(ii));       
            
            BB(1,1)=BB(1,1)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(1,jj),t_history(1,jj+1),Ki(ii),Si(ii));
            BB(1,2)=BB(1,2)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(2,jj),t_history(2,jj+1),Ki(ii),Si(ii));
            BB(1,3)=BB(1,3)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));
            BB(2,2)=BB(2,2)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(2,jj),t_history(2,jj+1),t_history(2,jj),t_history(2,jj+1),Ki(ii),Si(ii));
            BB(2,3)=BB(2,3)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(2,jj),t_history(2,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));
            BB(3,3)=BB(3,3)+wwn(t(Ind+1),t(jj),t(jj+1),t_history(3,jj),t_history(3,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));
            
            dBB(1,1)=dBB(1,1)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(1,jj),t_history(1,jj+1),Ki(ii),Si(ii));
            dBB(1,2)=dBB(1,2)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(2,jj),t_history(2,jj+1),Ki(ii),Si(ii));
            dBB(1,3)=dBB(1,3)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(1,jj),t_history(1,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));
            dBB(2,2)=dBB(2,2)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(2,jj),t_history(2,jj+1),t_history(2,jj),t_history(2,jj+1),Ki(ii),Si(ii));
            dBB(2,3)=dBB(2,3)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(2,jj),t_history(2,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));
            dBB(3,3)=dBB(3,3)+dwwn(t(Ind+1),t(jj),t(jj+1),t_history(3,jj),t_history(3,jj+1),t_history(3,jj),t_history(3,jj+1),Ki(ii),Si(ii));  
        end
        
        AA=AA-BB;                  %combine rotational and tidally changes
        dAA=dAA-dBB;
        
        AA(2,1)=AA(1,2);
        AA(3,1)=AA(1,3);
        AA(3,2)=AA(2,3);
        dAA(2,1)=dAA(1,2);
        dAA(3,1)=dAA(1,3);
        dAA(3,2)=dAA(2,3);   

        AA=AA-1/3*eye(3)*trace(AA);
        dAA=(dAA-1/3*eye(3)*trace(dAA))/ka;
        
        if Loadtype==1
            II=AA*R^5/3/GG+E;
            dII=dAA*R^5/3/GG+dE;
        else
            II=AA*R^5/3/GG+E*(t(Ind+1)-t0)/End_time;
            dE=E/End_time/ka;
            dII=dAA*R^5/3/GG+dE;
        end
        TT=t_history(:,Ind+1)/norm(t_history(:,Ind+1));
        RR=r_history(:,Ind+1)/norm(r_history(:,Ind+1));
        Q1=[TT,cross(RR,TT),RR];
%%     for rotational change   
        MoI1=Q1'*II*Q1;
        ddI=Q1'*dII*Q1;
        
        if Ind==1 && Iter==1
            I33_old=II(3,3)-E(3,3);
            I11_old=II(1,1)-E(1,1);
        end
        
        dMoI=MoI1;
        
        dI13=dMoI(1,3)+(MoI1(3,3)+I)*ddI(2,3)/Omega/(MoI1(3,3)-MoI1(2,2));
        th_new13=dI13/(MoI1(3,3)-MoI1(1,1));
        dI23=dMoI(2,3)-(MoI1(3,3)+I)*ddI(1,3)/Omega/(MoI1(3,3)-MoI1(1,1));
        th_new23=dI23/(MoI1(3,3)-MoI1(2,2));        
        th_new33=-(dMoI(3,3)-I33_old)/(dMoI(3,3)+I);
%%     for tidal change
        MoI2=-Q_T'*MoI1*Q_T;   
        ddI=-Q_T'*ddI*Q_T;
        dMoI=MoI2;
        
        dI13=dMoI(1,3)+(MoI2(3,3)+I)*ddI(2,3)/Omega/(MoI2(3,3)-MoI2(2,2));
        th_new13_T=dI13/(MoI2(3,3)-MoI2(1,1));
        dI23=dMoI(2,3)-(MoI2(3,3)+I)*ddI(1,3)/Omega/(MoI2(3,3)-MoI2(1,1));
        th_new23_T=dI23/(MoI2(3,3)-MoI2(2,2));        
        th_new33_T=-(dMoI(3,3)-I11_old)/(dMoI(3,3)+I);
%%     total change   
        X3=[th_new13,th_new23,1+th_new33]';
        Lodr(Ind+1)=norm(X3);
        X3=X3/Lodr(Ind+1);            
        
        X1=[1+th_new33_T,th_new23_T,-th_new13_T]';
        Lodt(Ind+1)=norm(X1);
        X1=X1/Lodt(Ind+1);               
        
        X2=cross(X3,X1);
        X2=X2/norm(X2);
        
%         X3=cross(X1,X2);  
        X1=cross(X2,X3);
        Q2=[X1,X2,X3];
%%             
        QQ=Q1*Q2;
        r_history(:,Ind+1)=QQ*[0,0,Omega]'*Lodr(Ind+1);  %%In the body-fixed coordinate now
        t_history(:,Ind+1)=QQ*[Omega*sqrt(T_C),0,0]'*Lodt(Ind+1);
        
        TT_old=TT;
        RR_old=RR;
        TT=t_history(:,Ind+1)/norm(t_history(:,Ind+1));
        RR=r_history(:,Ind+1)/norm(r_history(:,Ind+1));
%         Change=acos(sum(r_old.*r_history(:,Ind+1))/norm(r_old)/norm(r_history(:,Ind+1)));
%         if Change>=10/180*pi            
%             disp('Too fast angular change.')
%             return;
%         end

%         disp([num2str(Iter)  '    ' num2str(r_history(:,Ind+1)',10) '  '  num2str(t_history(:,Ind+1)',10)]);
        Iter=Iter+1;
        
        Improvement1=acos(sum(RR_old.*RR));       
        Improvement2=acos(sum(TT_old.*TT));    
    end    
    
%     AA_old=AA;
    Lc=QQ'*[x,y,z]'/R;
    Ls(:,Ind+1)=[asin(sqrt(Lc(1)^2+Lc(2)^2)),atan2(Lc(2),Lc(1))]';

    %     disp([num2str(Ind) '  ' num2str(Iter) '  ' num2str(r_history(:,Ind+1)')  num2str(t_history(:,Ind+1)')  '  ' num2str(Ls(1,Ind)*180/pi) '  ' num2str(Ls(2,Ind)*180/pi) ])
    disp([num2str(Ind) '  ' num2str(Iter,'%3d') ' ' num2str(Ls(1,Ind+1)*180/pi,8) '  ' num2str(Ls(2,Ind+1)*180/pi) ])

    %     if norm([pi/2-Ls(1,Ind+1),Ls(2,Ind+1)] )<5/180*pi
    %         Ind=Ind+1;
    %         break;
    %     end
    Body=AA*R^5/3/GG;
    [EIG,~]=eig(Body);
    epsilon(1,Ind)=acos(sum(EIG(:,3).*RR))*180/pi;
    epsilon(2,Ind)=acos(sum(EIG(:,1).*TT))*180/pi;
    if epsilon(1,Ind)>90
        epsilon(1,Ind)=180-epsilon(1,Ind);
    end
    if epsilon(2,Ind)>90
        epsilon(2,Ind)=180-epsilon(2,Ind);
    end
    Ind=Ind+1;
end
epsilon=epsilon(:,1:Ind-1);
t=t(2:Ind)-t(1);
r_history=r_history(:,2:Ind);
t_history=t_history(:,2:Ind);
Lodr=Lodr(2:Ind);
Lodt=Lodt(2:Ind);
Ls=Ls(:,2:Ind);
end

function wwns=wwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    Es0=exp(si*(t-T0));
    Es1=exp(si*(t-T1));
    dt=T1-T0;
    if abs(si)>1e-7
        wwns=2.*dt.^(-2).*((Es0-Es1)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)./si...
            +si.^(-1).*(Es0.*ki.*Wi0.*Wj0+(-1).*Es1.*ki.*Wi1.*Wj1)+...
            dt.^(-1).*(ki./si).*((Es1.*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1)+ ...
            Es0.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si);
    else
        %     wwns=(ki./si).*((-1+Es0-(t-T0).^2./dt.*si).*Wi0.*Wj0...
        %           +(1-Es1-(t-T1).^2./dt.*si).*Wi1.*Wj1...
        %           +((t-T0).^2+(t-T1).^2)./dt.*si.*(Wi1.*Wj0+Wi0.*Wj1));
        
        %         b00=(1/2).*dt.^(-2).*ki.*si.^(-2).*((-2).*dt.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0))+dt.^2.*si.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0))+(-2).*dt.*((-2)+si.*((-2).*t+T0+T1)));
        %         b10=(1/2).*dt.^(-2).*ki.*si.^(-2).*(2.*dt.*((-2)+si.*((-2).*t+T0+T1))+dt.*(4+si.*(4.*t+(-2).*(T0+T1))+si.^2.*(2.*t.^2+T0.^2+T1.^2+(-2).*t.*(T0+T1))));
        %         b11=(-1/6).*dt.^(-2).*ki.*si.^(-2).*(2.*dt.*(6+si.*(6+si.*(3+si.*(t+(-1).*T1)).*(t+(-1).*T1)).*(t+(-1).*T1))+dt.^2.*si.*(6+si.*(6+si.*(3+si.*(t+(-1).*T1)).*(t+(-1).*T1)).*(t+(-1).*T1))+2.*(T0+(-1).*T1) ...
        %             .*(6+si.*(6.*t+(-3).*(T0+T1))+si.^2.*(3.*t.^2+T0.^2+T0.*T1+T1.^2+( -3).*t.*(T0+T1))));
        
        b10=(ki./si).*((1/2).*dt.^(-1).*si.*((t+(-1).*T0).^2+(t+(-1).*T1).^2)+dt.^(-2).* si.^(-1).*(2.*dt+2.*(T0+(-1).*T1))+dt.^(-2).*(dt.*(2.*t+(-1).*T0+(-1).*T1)+((-1).*T0+T1).*((-2).*t+T0+T1)));
        b00=(ki./si).*((1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T0)+(-2).*dt.*(t+(-1).*T0).^2)+(1/2).*si.^2.*(t+(-1).*T0).^2+(1/2).*dt.^(-2).*si.^(-1).*(...
            (-4).*dt+(-4).*(T0+(-1).*T1))+(1/2).*dt.^(-2).*(2.*dt.^2+(-4).* dt.*(t+(-1).*T0)+2.*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        b11=(ki./si).*((-1/2).*dt.^(-2).*si.*(2.*dt.^2.*(t+(-1).*T1)+2.*dt.*(t+(-1).*T1).^2)+(-1/2).*dt.^(-2).*si.^(-1).*(4.*dt+4.*(T0+(-1).*T1))+(-1/2).* ...
            si.^2.*(t+(-1).*T1).^2+(-1/2).*dt.^(-2).*(2.*dt.^2+4.*dt.*(t+(-1).*T1)+(-2).*(T0+(-1).*T1).*((-2).*t+T0+T1)));
        
        %         b00=(1/2).*dt.^(-2).*ki.*si.^(-2).*((-2).*dt.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0))+dt.^2.*si.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+( -1).*T0))+2.*(T0+(-1).*T1).*((-2)+si.*((-2).*t+T0+T1)));
        %         b10=(1/2).*dt.^(-2).*ki.*si.^(-2).*((-2).*(T0+(-1).*T1).*((-2)+si.*((-2).*t+T0+T1))+dt.*(4+si.*(4.*t+(-2).*(T0+T1))+si.^2.*(2.*t.^2+  T0.^2+T1.^2+(-2).*t.*(T0+T1))));
        %         b11=(1/2).*dt.^(-2).*ki.*si.^(-2).*((-1).*dt.*(2+dt.*si).*(2+si.*t.*(2+si.*t))+(-4).*(1+si.*t).*T0+2.*si.*T0.^2+2.*(2+dt.*si.*(2+dt.*si)).*(1+si.*t).*T1+(-1).*si.*(2+dt.*si.*(2+dt.*si)).*T1.^2);
        wwns=b00.*Wi0.*Wj0+b10.*(Wi1.*Wj0+Wi0.*Wj1)+b11.*Wi1.*Wj1;
    end
else    
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si*dt(1))>1e-10
        wwns=2.*dt.^(-2).*((-1+Esi)./si)*(ki./si).*((Wi0- Wi1).*(Wj0-Wj1)/si)...
            +(ki./si).*(Esi.*Wi0.*Wj0-Wi1.*Wj1)+...
            dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+ ...
            Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)))./si;
    else
        wwns=(ki./si).*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0-Wi1.*Wj1);
%         b00=(1/2).*dt.^(-2).*ki.*si.^(-2).*((-2).*dt.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0))+dt.^2.*si.*(2+si.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0))+2.*(2+si.*(t+(-1).*T0)).*(t+(-1).*T0));
%         b10=(1/2).*dt.^(-2).*ki.*si.^(-2).*(dt.*(2+si.*(2+si.*(t+(-1).*T0)).* (t+(-1).*T0))+(-2).*si.*(t+(-1).*T0).^2+2.*((-2).*t+T0+T1));
%         b11=dt.^(-2).*ki.*si.^(-2).*(t+(-1).*T1).*(2+si.*(t+(-2).*T0+T1));
%         wwns=b00.*Wi0.*Wj0+b10.*(Wi1.*Wj0+Wi0.*Wj1)+b11.*Wi1.*Wj1;
    end
end
end

function dwwns=dwwn(t,T0,T1,Wi0,Wi1,Wj0,Wj1,ki,si)
if t>T1
    dt=T1-T0;
    if abs(si)>1e-10
        dwwns=ki./(si^2*dt.^2).*(exp(-si*T0+t*si).*(Wi1.*((-2+si*dt).*Wj0+2*Wj1)+Wi0.*((2-si*(2-si*dt).*dt).*Wj0...
            +(-2+si*dt).*Wj1))-...
            exp(si*(t-T1)).*(Wi0.*(2*Wj0+(-2-si*dt).*Wj1)+Wi1.*(2*Wj1+(-2-si*dt).*(Wj0-si*dt.*Wj1))));
    else
        Es0=exp(si*(t-T0));
%         Es1=exp(si*(t-T1));
        dwwns=ki.*Es0.*((dt.*si./2).*(Wi1.*Wj0+Wi0.*Wj1)+(1-exp(-dt.*si)-2*dt.*si).* Wi1.*Wj1);
    end
else
    dt=T1-T0;
    Esi=exp(dt.*si);
    if abs(si)>1e-10
        dwwns=Esi.*ki.*Wi0.*Wj0...
            +2.*dt.^(-2).*((-1+Esi)./si).*(ki./si).*(Wi0+(-1).*Wi1).*(Wj0+(-1).*Wj1)...
            +dt.^(-1).*(ki./si).*(Wi1.*(Wj0+(-2).*Wj1)+Wi0.*Wj1+Esi.*(Wi1.*Wj0+Wi0.*((-2).*Wj0+Wj1)));
    else
        dwwns=ki.*(Wi1.*Wj0+Wi0.*Wj1-Wi0.*Wj0);
    end        
end   
end


function Q=RotationM(X, Coord) %X in spherical coordinate (theta,nabda,R). Coord: 1 spherical in rad; 2 Cartesian.
    if Coord==1
        th=zeros(3);
        th(1)=sin(X(1))*cos(X(2));
        th(2)=sin(X(1))*sin(X(2));
        th(3)=cos(X(1));
%         th=th/norm(th);
    else
        X=X/norm(X);
        th=X;
    end
    Q=[th(3)+th(2)^2/(1+th(3)),-th(1)*th(2)/(1+th(3)),-th(1); -th(1)*th(2)/(1+th(3)),1-th(2)^2/(1+th(3)),-th(2);th(1),th(2),th(3)]';
end